from typing import List
from cfn_guard_test.suite import CfnGuardTestSuite


class CfnGuardTestSuites:
    """
    Understands a cfn-guard test suites.

    A cfn-guard test suites can contain 1 or more cfn-guard test suite. This is a wrapper on the highest level.
    """

    __suites: List[CfnGuardTestSuite]

    def __init__(self) -> None:
        self.__suites = []

    def add_test_suite(self, case: CfnGuardTestSuite) -> None:
        self.__suites.append(case)

    @property
    def errors(self) -> int:
        return sum(map(lambda suite: suite.errors, self.error_suites), 0)

    @property
    def passed(self) -> int:
        return sum(map(lambda suite: suite.passed, self.passed_suites), 0)

    @property
    def failed(self) -> int:
        return sum(map(lambda suite: suite.failed, self.failed_suites), 0)

    @property
    def all_suites(self) -> List[CfnGuardTestSuite]:
        return self.__suites

    @property
    def failed_suites(self) -> List[CfnGuardTestSuite]:
        return list(filter(lambda suite: suite.failed, self.__suites))

    @property
    def error_suites(self) -> List[CfnGuardTestSuite]:
        return list(filter(lambda suite: suite.errors, self.__suites))

    @property
    def error_messages(self) -> List[str]:
        messages = []
        list(
            map(lambda suite: messages.extend(suite.error_messages), self.error_suites)
        )
        return messages

    @property
    def failure_messages(self) -> List[str]:
        messages = []

        def extend(suite: CfnGuardTestSuite) -> None:
            messages.extend(suite.failed_test_cases_messages)

        list(map(extend, self.failed_suites))
        return messages

    @property
    def passed_suites(self) -> List[CfnGuardTestSuite]:
        return list(filter(lambda suite: suite.passed, self.__suites))



from typing import List

from cfn_guard_test.rule import CfnGuardRule
from cfn_guard_test.case import CfnGuardTestCase, ErrorTestCase
from cfn_guard_test.suites import CfnGuardTestSuites
from junit_xml import TestCase, TestSuite, to_xml_report_string


class CfnGuardReport:
    """
    Understands how to create reports
    """

    __suites: List[TestSuite]

    def __init__(self, suites: CfnGuardTestSuites):
        self.__suites = []

        for suite in suites.all_suites:
            self.__timestamp = suite.duration

            self.__suites.append(
                TestSuite(
                    name=suite.ruleset, test_cases=self.__get_all_test_cases(suite)
                )
            )

    @property
    def elapsed_sec(self) -> float:
        """
        Because we only have the time for the whole suite we will only return this once to the case.
        """
        timestamp = 0.0

        if self.__timestamp:
            timestamp = self.__timestamp
            self.__timestamp = 0.0

        return timestamp

    def __get_all_test_cases(self, suite):
        cases = []

        for case in suite.all_test_cases:
            if case.errors:
                cases.append(self.__create_error(case))

            for rule in case.all_rules:
                cases.append(self.__create_test_case(case=case, rule=rule))

        return cases

    def __create_error(self, error: ErrorTestCase) -> TestCase:
        test_case = TestCase(name=error.name, elapsed_sec=self.elapsed_sec)
        test_case.add_error_info(message=error.message, output=error.details)

        return test_case

    def __create_test_case(
        self, case: CfnGuardTestCase, rule: CfnGuardRule
    ) -> TestCase:
        test_case = TestCase(
            name=rule.name, classname=case.name, elapsed_sec=self.elapsed_sec
        )

        if rule.skipped:
            test_case.add_skipped_info(
                f'Rule {rule.name} was skipped on case #{case.number} "{case.name}"',
                output=case.details,
            )

        if rule.failed:
            test_case.add_failure_info(
                f'Rule {rule.name} failed on case #{case.number} "{case.name}"',
                output=case.details,
            )

        return test_case

    def write(self, path: str) -> None:
        with open(path, "w") as f:
            f.write(to_xml_report_string(self.__suites, prettyprint=True))



from typing import List
from cfn_guard_test.case import CfnGuardTestCase


class CfnGuardTestSuite:
    """
    Understands a cfn-guard test suite

    A single cfn-guard test suite can contain 1 or more cfn-guard test cases. A cfn-guard test suite contains are all
    cases defined in a single `<name>_tests.yaml`.
    """

    __name: str
    __cases: List[CfnGuardTestCase]

    def __init__(self, name: str, duration: float) -> None:
        self.__name = name
        self.__duration = duration
        self.__cases = []

    def add_test_case(self, case: CfnGuardTestCase) -> None:
        self.__cases.append(case)

    @property
    def ruleset(self) -> str:
        return self.__name

    @property
    def duration(self) -> float:
        return self.__duration

    @property
    def errors(self) -> int:
        rules = self.error_test_cases
        return len(rules) if rules else 0

    @property
    def passed(self) -> int:
        rules = self.passed_test_cases
        return len(rules) if rules else 0

    @property
    def failed(self) -> int:
        rules = self.failed_test_cases
        return len(rules) if rules else 0

    @property
    def all_test_cases(self) -> List[CfnGuardTestCase]:
        return self.__cases

    @property
    def error_test_cases(self) -> List[CfnGuardTestCase]:
        return list(filter(lambda case: case.errors, self.__cases))

    @property
    def failed_test_cases(self) -> List[CfnGuardTestCase]:
        return list(filter(lambda case: case.failed, self.__cases))

    @property
    def error_messages(self) -> List[str]:
        messages = []
        list(
            map(
                lambda case: messages.append(case.message + "\n\n" + case.details),
                self.error_test_cases,
            )
        )
        return messages

    @property
    def failed_test_cases_messages(self) -> List[str]:
        messages = []

        def extend(case: CfnGuardTestCase) -> None:
            messages.append(case.message)
            messages.extend(case.failed_rules_messages(suite_name=self.ruleset))

        list(map(extend, self.failed_test_cases))
        return list(filter(None, messages))

    @property
    def passed_test_cases(self) -> List[CfnGuardTestCase]:
        return list(filter(lambda case: case.passed, self.__cases))



from typing import List
from cfn_guard_test.rule import CfnGuardRule


class CfnGuardTestCase:
    """
    Understands a cfn-guard test case

    A single cfn-guard test case can contain 1 ore more rules that are evaluated. Each definition in the
    `<name>_tests.yaml` file results in a cfn-guard test case.
    """

    __name: str
    __number: int
    __rules: List[CfnGuardRule]

    def __init__(self, name: str, number: int) -> None:
        self.__name = name
        self.__number = number
        self.__rules = []

    def add_rule(self, rule: CfnGuardRule) -> None:
        self.__rules.append(rule)

    @property
    def name(self) -> str:
        return self.__name

    @property
    def number(self) -> int:
        return self.__number

    @property
    def message(self) -> str:
        return ""

    @property
    def details(self) -> str:
        return ""

    @property
    def passed(self) -> int:
        rules = self.passed_rules
        return len(rules) if rules else 0

    @property
    def errors(self) -> int:
        return 0

    @property
    def failed(self) -> int:
        rules = self.failed_rules
        return len(rules) if rules else 0

    @property
    def all_rules(self) -> List[CfnGuardRule]:
        return self.__rules

    @property
    def failed_rules(self) -> List[CfnGuardRule]:
        return list(filter(lambda case: case.failed, self.__rules))

    def failed_rules_messages(self, suite_name: str) -> List[str]:
        messages: List[str] = []

        def extend(rule: CfnGuardRule) -> None:
            messages.append(
                rule.failed_message(
                    suite_name=suite_name, case_number=self.number, case_name=self.name
                )
            )

        list(map(extend, self.failed_rules))
        return messages

    @property
    def passed_rules(self) -> List[CfnGuardRule]:
        return list(filter(lambda case: case.passed, self.__rules))


class ErrorTestCase(CfnGuardTestCase):
    def __init__(self, message: str, details: str) -> None:
        super().__init__(name="Loading Error", number=1)
        self.__message = message
        self.__details = details

    @property
    def errors(self) -> int:
        return 1

    @property
    def message(self) -> str:
        return self.__message

    @property
    def details(self) -> str:
        return self.__details
